import pandas as pd
import matplotlib.pyplot as plt
from keras.models import Sequential
from keras.layers import Dense
from sklearn.metrics import ConfusionMatrixDisplay
from sklearn.metrics import confusion_matrix
import tensorflow as tf
#Do a simple ANN script using tensorflow
#SM Sep 2022

#just for GPU usage / if you are using GPU processing you might need this
gpus = tf.config.experimental.list_physical_devices('GPU')
if gpus:
  try:
    tf.config.experimental.set_virtual_device_configuration(
        gpus[0],[tf.config.experimental.VirtualDeviceConfiguration(memory_limit=2048)])
  except RuntimeError as e:
    print(e)

#determine your training and testing datasets and read them (usually, ex. 2/3 train, 1/3 test)
#use the ISO argument if you have special characters otherwise default is UTF-8
train_data = pd.read_csv('LIFE_TRAIN.csv', encoding = "ISO-8859-1")
test_data = pd.read_csv('LIFE_TEST.csv', encoding = "ISO-8859-1")

#determine your features ie. which columns are important to predict?
features = ['under_five_deaths', 'diphtheria', 'status']

#X is your features that will be used, y is your target you want to predict
#get_dummies will change categorical variables into numbers for calculation
X_train = pd.get_dummies(train_data[features])
y_train = train_data['highexpectancy']
X_test = pd.get_dummies(test_data[features])
y_test = test_data['highexpectancy']

#load a variable with the number of features
input_dim = len(X_train.columns)

#specify neurons, epochs, and model type
neurons = 24
epochs = 50
model = Sequential()

#add one layer with specified neurons, and one output layer
#use sigmoid and binary_crossentropy because we predict 1 or 0
model.add(Dense(neurons, input_dim=input_dim, activation='relu'))
model.add(Dense(1, activation='sigmoid'))
model.compile(loss='binary_crossentropy', optimizer='adam', metrics=['accuracy'])

#fit our model to the data and record in a history variable
history = model.fit(X_train, y_train, epochs=epochs, batch_size=100, verbose=1, validation_split=0.33)
#test our model using the test dataset
predictions = model.predict(X_test)

#set our predictions to strictly 1 or 0
predictions = (predictions > 0.5)*1

#put our output in a csv file to open and compare
output = pd.DataFrame(
    {'ID': test_data.ID,
     'highexpectancy': predictions.flatten()})
output.to_csv('life_exp.csv', index = False)

#evaluate our scores by running our model on the test data
scores = model.evaluate(X_test, y_test, verbose=0)
print(scores)

#make a confusion matrix
cfm = confusion_matrix(y_test, predictions)
disp = ConfusionMatrixDisplay(cfm)
disp.plot()
plt.gca().invert_yaxis()
plt.show()
#plot accuracy
plt.plot(history.history['accuracy'])
plt.plot(history.history['val_accuracy'])
plt.title('model accuracy')
plt.ylabel('accuracy')
plt.xlabel('epoch')
plt.legend(['train', 'test'], loc='upper left')
plt.show()
#plot loss
plt.plot(history.history['loss'])
plt.plot(history.history['val_loss'])
plt.title('model loss')
plt.ylabel('loss')
plt.xlabel('epoch')
plt.legend(['train', 'test'], loc='upper left')
plt.show()


